@@ -18,9 +18,9 @@
 
 """Find decompacted total sediment thickness and tectonic subsidence through time.
 
-:func:`backstrip_well` finds decompacted total sediment thickness and tectonic subsidence for each age in a well.
+:func:`pybacktrack.backstrip_well` finds decompacted total sediment thickness and tectonic subsidence for each age in a well.
 
-:func:`write_well` writes decompacted parameters as columns in a text file.
+:func:`pybacktrack.write_backstrip_well` writes decompacted parameters as columns in a text file.
 """
 
 
@@ -245,12 +245,7 @@ def backstrip_well(
     # that is an average over the decompacted surface layer's period of deposition.
     if sea_level_model:
         # Create sea level object for integrating sea level over time periods.
-        #
-        # If a sea level *model name* was specified then create it from a bundled sea level model.
-        if sea_level_model in pybacktrack.bundle_data.BUNDLE_SEA_LEVEL_MODEL_NAMES:
-            sea_level = SeaLevel.create_from_bundled_model(sea_level_model)
-        else:
-            sea_level = SeaLevel(sea_level_model)
+        sea_level = SeaLevel.create_from_model_or_bundled_model_name(sea_level_model)
         
         # The sea level (relative to present day) is integrated over the period of deposition of each
         # stratigraphic layer (in decompacted wells) and added as a 'sea_level' attribute to each decompacted well.
@@ -429,23 +424,14 @@ def write_well(
                     average_decompacted_density = decompacted_well.get_average_decompacted_density()
                     column_str = column_float_format_string.format(average_decompacted_density, width=column_width)
                 elif decompacted_column == COLUMN_AVERAGE_TECTONIC_SUBSIDENCE:
-                    min_tectonic_subsidence, max_tectonic_subsidence = decompacted_well.get_min_max_tectonic_subsidence_from_water_depth(
-                        decompacted_well.min_water_depth,
-                        decompacted_well.max_water_depth,
-                        decompacted_well.get_sea_level(None))  # Is None if no sea level model was specified
+                    min_tectonic_subsidence, max_tectonic_subsidence = decompacted_well.get_min_max_tectonic_subsidence()
                     average_tectonic_subsidence = (min_tectonic_subsidence + max_tectonic_subsidence) / 2.0
                     column_str = column_float_format_string.format(average_tectonic_subsidence, width=column_width)
                 elif decompacted_column == COLUMN_MIN_TECTONIC_SUBSIDENCE:
-                    min_tectonic_subsidence, max_tectonic_subsidence = decompacted_well.get_min_max_tectonic_subsidence_from_water_depth(
-                        decompacted_well.min_water_depth,
-                        decompacted_well.max_water_depth,
-                        decompacted_well.get_sea_level(None))  # Is None if no sea level model was specified
+                    min_tectonic_subsidence, max_tectonic_subsidence = decompacted_well.get_min_max_tectonic_subsidence()
                     column_str = column_float_format_string.format(min_tectonic_subsidence, width=column_width)
                 elif decompacted_column == COLUMN_MAX_TECTONIC_SUBSIDENCE:
-                    min_tectonic_subsidence, max_tectonic_subsidence = decompacted_well.get_min_max_tectonic_subsidence_from_water_depth(
-                        decompacted_well.min_water_depth,
-                        decompacted_well.max_water_depth,
-                        decompacted_well.get_sea_level(None))  # Is None if no sea level model was specified
+                    min_tectonic_subsidence, max_tectonic_subsidence = decompacted_well.get_min_max_tectonic_subsidence()
                     column_str = column_float_format_string.format(max_tectonic_subsidence, width=column_width)
                 elif decompacted_column == COLUMN_AVERAGE_WATER_DEPTH:
                     # Use extra attributes (min/max water depth) loaded into original well...
