@@ -115,15 +115,13 @@ def backtrack_well(
         Crustal thickness filename.
         Used to obtain crustal thickness at well location.
     dynamic_topography_model : string or tuple, optional
-        Represents a time-dependent dynamic topography raster grid.
-        Currently only used for oceanic floor (ie, well location inside age grid)
-        it is not used if well is on continental crust (passive margin).
+        Represents a time-dependent dynamic topography raster grid (in *mantle* frame).
         
         Can be either:
         
         * A string containing the name of a bundled dynamic topography model.
         
-          Choices include ``terra``, ``M1``, ``M2``, ``M3``, ``M4``, ``M5``, ``M6``, ``M7``, ``ngrand``, ``s20rts`` and ``smean``.
+          Choices include ``terra``, ``M1``, ``M2``, ``M3``, ``M4``, ``M5``, ``M6``, ``M7``, ``ngrand``, ``s20rts``, ``smean``, ``AY18`` and ``KM16``.
         * A tuple containing the three elements (dynamic topography list filename, static polygon filename, rotation filenames).
         
           The first tuple element is the filename of file containing list of dynamic topography grids (and associated times).
@@ -144,7 +142,7 @@ def backtrack_well(
         The stratigraphic units in the well might not record the full depth of sedimentation.
         The base unit covers the remaining depth from bottom of well to the total sediment thickness.
         Defaults to ``Shale``.
-    ocean_age_to_depth_model : {pybacktrack.AGE_TO_DEPTH_MODEL_GDH1, pybacktrack.AGE_TO_DEPTH_MODEL_CROSBY_2007} or function, optional
+    ocean_age_to_depth_model : {pybacktrack.AGE_TO_DEPTH_MODEL_RHCW18, pybacktrack.AGE_TO_DEPTH_MODEL_CROSBY_2007, pybacktrack.AGE_TO_DEPTH_MODEL_GDH1} or function, optional
         The model to use when converting ocean age to depth at well location
         (if on ocean floor - not used for continental passive margin).
         It can be one of the enumerated values, or a callable function accepting a single non-negative age parameter and returning depth (in metres).
@@ -152,9 +150,9 @@ def backtrack_well(
         Optional time period of rifting (if on continental passive margin - not used for oceanic floor).
         If specified then should be a 2-tuple (rift_start_age, rift_end_age) where rift_start_age can be None
         (in which case rifting is considered instantaneous from a stretching point-of-view, not thermal).
-        If specified then overrides value in well file.
-        If well is on continental passive margin then at least rift end age should be specified
-        either here or in well file.
+        If specified then overrides value in well file (and value from builtin rift start/end grids).
+        If well is on continental passive margin then at least rift end age should be specified either here or in well file, or
+        well location must be inside rifting region of builtin rift start/end grids, otherwise a ValueError exception will be raised.
     well_location : tuple, optional
         Optional location of well.
         If not provided then is extracted from the ``well_filename`` file.
@@ -182,6 +180,9 @@ def backtrack_well(
         If ``lithology_column`` is not the largest column number (must be last column).
     ValueError
         If ``well_location`` is not specified *and* the well location was not extracted from the well file.
+    ValueError
+        If well is on continental passive margin but rift end age was not specified by user and was not extracted from well file,
+        and well location was not inside rifting region of builtin rift start/end grids.
     
     Notes
     -----
@@ -208,7 +209,6 @@ def backtrack_well(
     well = load_well(
         well_filename,
         lithologies,
-        rifting_period,
         well_location,
         well_bottom_age_column,
         well_bottom_depth_column,
@@ -231,13 +231,33 @@ def backtrack_well(
         # (so they specified None for 'age_grid_filename').
         age = None
     
-    # If well is on continental passive margin then rift end age needs to be
-    # specified by user or obtained from well file.
+    # If well is on continental passive margin then rift end age needs to be specified by user or
+    # obtained from well file or from builtin rift start/end grids (prioritized in that order).
     if age is None:
-        if well.rift_end_age is None:
-            raise ValueError('Well is on continental passive margin but rift end age was '
-                             'not extracted from well file and was not specified by user. '
-                             'Either add RiftEndAge to the well file or specify rift end age on command-line.')
+        # If the rifting period was specified then override the value read from the well file (if read) and builtin grids.
+        # The rift end time must be provided but the rift start time is optional.
+        if rifting_period is not None:
+            rift_start_age, rift_end_age = rifting_period
+            if rift_end_age is None:
+                raise ValueError('If rifting period is specified then rifting end time must not be None')
+            well.rift_start_age, well.rift_end_age = rift_start_age, rift_end_age
+        elif well.rift_start_age is not None or well.rift_end_age is not None:
+            # The well file has provided the rift end time (and optional rift start time).
+            if well.rift_end_age is None:
+                raise ValueError('Well file provides a rift start age but not a rift end age')
+        else:
+            # Attempt to get rift start/end from builtin rift start/end grids.
+            rift_end_age = _sample_grid(well.longitude, well.latitude, pybacktrack.bundle_data.BUNDLE_RIFTING_END_FILENAME)
+            if math.isnan(rift_end_age):
+                rift_end_age = None
+            if rift_end_age is None:
+                raise ValueError('Well is on continental passive margin but rift end age was not specified by user and was not extracted from well file, '
+                                'and well location was not inside rifting region of builtin rift start/end grids. '
+                                'Either specify rift end age (on command-line) or add RiftEndAge to the well file.')
+            rift_start_age = _sample_grid(well.longitude, well.latitude, pybacktrack.bundle_data.BUNDLE_RIFTING_START_FILENAME)
+            if math.isnan(rift_start_age):
+                rift_start_age = None
+            well.rift_start_age, well.rift_end_age = rift_start_age, rift_end_age
     
     # Sample topography grid at well location.
     present_day_topography = _sample_grid(well.longitude, well.latitude, topography_filename)
@@ -313,15 +333,7 @@ def backtrack_well(
     
     # Create time-dependent grid object for sampling dynamic topography (if requested).
     if dynamic_topography_model:
-        # If a dynamic topography *model name* was specified then create it from a bundled dynamic topography model.
-        if isinstance(dynamic_topography_model, str if sys.version_info[0] >= 3 else basestring):  # Python 2 vs 3.
-            dynamic_topography = DynamicTopography.create_from_bundled_model(dynamic_topography_model, well.longitude, well.latitude, age)
-        else:
-            # Otherwise we're expecting a 3-tuple.
-            dynamic_topography_list_filename, dynamic_topography_static_polygon_filename, dynamic_topography_rotation_filenames = dynamic_topography_model
-            dynamic_topography = DynamicTopography(
-                dynamic_topography_list_filename, dynamic_topography_static_polygon_filename, dynamic_topography_rotation_filenames,
-                well.longitude, well.latitude, age)
+        dynamic_topography = DynamicTopography.create_from_model_or_bundled_model_name(dynamic_topography_model, well.longitude, well.latitude, age)
     else:
         dynamic_topography = None
     
@@ -351,7 +363,6 @@ def backtrack_well(
 def load_well(
         well_filename,
         lithologies,
-        rifting_period=None,
         well_location=None,
         well_bottom_age_column=0,
         well_bottom_depth_column=1,
@@ -363,13 +374,6 @@ def load_well(
     
     lithologies: a dict mapping lithology names to lithology.Lithology objects.
     
-    rifting_period: Optional time period of rifting (if on continental passive margin - not used for oceanic floor).
-                    If specified then should be a 2-tuple (rift_start_age, rift_end_age) where rift_start_age can be None
-                    (in which case rifting is considered instantaneous from a stretching point-of-view, not thermal).
-                    If specified then overrides value in well file.
-                    If well is on continental passive margin then at least rift end age should be specified
-                    either here or in well file.
-    
     well_location: Optional location of well. If not provided then is extracted from 'well_filename' file.
                    If specified then overrides value in well file.
                    If specified then must be a 2-tuple (longitude, latitude) in degrees.
@@ -434,14 +438,6 @@ def load_well(
     if well.longitude is None or well.latitude is None:
         raise ValueError('Well location was not extracted from well file and was not specified by user.')
     
-    # If the rifting period was specified then override the value read from the well file (if read).
-    if rifting_period is not None:
-        rift_start_age, rift_end_age = rifting_period
-        if rift_start_age is not None:
-            well.rift_start_age = rift_start_age
-        if rift_end_age is not None:
-            well.rift_end_age = rift_end_age
-    
     return well
 
 
@@ -519,12 +515,7 @@ def _add_sea_level(
     """
     
     # Create sea level object for integrating sea level over time periods.
-    #
-    # If a sea level *model name* was specified then create it from a bundled sea level model.
-    if sea_level_model in pybacktrack.bundle_data.BUNDLE_SEA_LEVEL_MODEL_NAMES:
-        sea_level = SeaLevel.create_from_bundled_model(sea_level_model)
-    else:
-        sea_level = SeaLevel(sea_level_model)
+    sea_level = SeaLevel.create_from_model_or_bundled_model_name(sea_level_model)
     
     for decompacted_well in decompacted_wells:
         decompacted_well.sea_level = sea_level.get_average_level(
@@ -574,7 +565,7 @@ def _add_oceanic_tectonic_subsidence(
     # Assume this offset is constant for all ages and use it to adjust the subsidence obtained from age-to-depth model for other ages.
     tectonic_subsidence_model_adjustment = present_day_tectonic_subsidence - present_day_tectonic_subsidence_from_model
     
-    # Get present-day dynamic topography (if requested).
+    # Get present-day dynamic topography (if we have dynamic topography).
     if dynamic_topography:
         dynamic_topography_at_present_day = dynamic_topography.sample(0.0)
     
@@ -615,7 +606,7 @@ def _add_continental_tectonic_subsidence(
     to each decompacted well.
     """
     
-    # Get dynamic topography (if requested) at rift start and remove contribution of dynamic topography
+    # Get dynamic topography (if we have dynamic topography) at rift start and remove contribution of dynamic topography
     # to subsidence at present day so we can estimate subsidence due to stretching and thermal effects only.
     if dynamic_topography:
         dynamic_topography_at_present_day = dynamic_topography.sample(0.0)
@@ -966,7 +957,7 @@ def backtrack_and_write_well(
         
         * A string containing the name of a bundled dynamic topography model.
         
-          Choices include ``terra``, ``M1``, ``M2``, ``M3``, ``M4``, ``M5``, ``M6``, ``M7``, ``ngrand``, ``s20rts`` and ``smean``.
+          Choices include ``terra``, ``M1``, ``M2``, ``M3``, ``M4``, ``M5``, ``M6``, ``M7``, ``ngrand``, ``s20rts``, ``smean``, ``AY18`` and ``KM16``.
         * A tuple containing the three elements (dynamic topography list filename, static polygon filename, rotation filenames).
         
           The first tuple element is the filename of file containing list of dynamic topography grids (and associated times).
@@ -987,7 +978,7 @@ def backtrack_and_write_well(
         The stratigraphic units in the well might not record the full depth of sedimentation.
         The base unit covers the remaining depth from bottom of well to the total sediment thickness.
         Defaults to ``Shale``.
-    ocean_age_to_depth_model : {pybacktrack.AGE_TO_DEPTH_MODEL_GDH1, pybacktrack.AGE_TO_DEPTH_MODEL_CROSBY_2007} or function, optional
+    ocean_age_to_depth_model : {pybacktrack.AGE_TO_DEPTH_MODEL_RHCW18, pybacktrack.AGE_TO_DEPTH_MODEL_CROSBY_2007, pybacktrack.AGE_TO_DEPTH_MODEL_GDH1} or function, optional
         The model to use when converting ocean age to depth at well location
         (if on ocean floor - not used for continental passive margin).
         It can be one of the enumerated values, or a callable function accepting a single non-negative age parameter and returning depth (in metres).
@@ -995,9 +986,9 @@ def backtrack_and_write_well(
         Optional time period of rifting (if on continental passive margin - not used for oceanic floor).
         If specified then should be a 2-tuple (rift_start_age, rift_end_age) where rift_start_age can be None
         (in which case rifting is considered instantaneous from a stretching point-of-view, not thermal).
-        If specified then overrides value in well file.
-        If well is on continental passive margin then at least rift end age should be specified
-        either here or in well file.
+        If specified then overrides value in well file (and value from builtin rift start/end grids).
+        If well is on continental passive margin then at least rift end age should be specified either here or in well file, or
+        well location must be inside rifting region of builtin rift start/end grids, otherwise a ValueError exception will be raised.
     decompacted_columns : list of columns, optional
         The decompacted columns (and their order) to output to ``decompacted_wells_filename``.
         
@@ -1035,6 +1026,9 @@ def backtrack_and_write_well(
         If ``lithology_column`` is not the largest column number (must be last column).
     ValueError
         If ``well_location`` is not specified *and* the well location was not extracted from the well file.
+    ValueError
+        If well is on continental passive margin but rift end age was not specified by user and was not extracted from well file,
+        and well location was not inside rifting region of builtin rift start/end grids.
     
     Notes
     -----
@@ -1230,7 +1224,6 @@ def main():
             
             setattr(namespace, self.dest, (grid_list_filename, static_polygons_filename, rotation_filenames))
 
-    ocean_age_to_depth_model_dict = dict((model_name, model) for model, model_name, _ in age_to_depth.ALL_MODELS)
     ocean_age_to_depth_model_name_dict = dict((model, model_name) for model, model_name, _ in age_to_depth.ALL_MODELS)
     default_ocean_age_to_depth_model_name = ocean_age_to_depth_model_name_dict[age_to_depth.DEFAULT_MODEL]
     
@@ -1314,18 +1307,18 @@ def main():
         '-rs', '--rift_start_time', type=argparse_non_negative_float,
         metavar='rift_start_time',
         help='Optional start time of rifting (in My). '
-             'Only used if well is located on continental passive margin (outside age grid), '
-             'in which case it is not required (even if also not provided inside the well file as '
-             '"# RiftStartTime = <rift_start_time>") because it will essentially default to '
-             'the rift "end" time. However providing a start time will result in more accurate '
-             'subsidence values generated during rifting.')
+             'Only used if well is located on continental passive margin (outside age grid). '
+             'If not specified (but "rift_end_time" is specified) then it defaults to "rift_end_time". '
+             'However providing a rift start time will result in more accurate subsidence values generated during rifting. '
+             'If it is specified then "rift_end_time" must also be specified.')
     parser.add_argument(
         '-re', '--rift_end_time', type=argparse_non_negative_float,
         metavar='rift_end_time',
         help='Optional end time of rifting (in My). '
-             'Only used if well is located on continental passive margin (outside age grid), '
-             'in which case it must be specified if it is not provided inside the well file '
-             '(as "# RiftEndTime = <rift_end_time>"). Overrides well file if both specified.')
+             'Only used if well is located on continental passive margin (outside age grid), in which case it '
+             'must be specified if it is not provided inside the well file (as "# RiftEndTime = <rift_end_time>") '
+             'and well location is not inside rifting region of builtin rift start/end grids. '
+             'Overrides well file (and builtin rifting start/end grids) if specified.')
     
     parser.add_argument(
         '-o', '--output_well_filename', type=argparse_unicode,
@@ -1397,6 +1390,7 @@ def main():
              'Can be used both for oceanic floor and continental passive margin '
              '(ie, well location inside or outside age grid). '
              'First filename contains a list of dynamic topography grids (and associated times). '
+             'Note that each grid must be in the mantle reference frame. '
              'Second filename contains static polygons associated with dynamic topography model '
              '(used to assign plate ID to well location so it can be reconstructed). '
              'Third filename (and optional fourth, etc) are the rotation files associated with model '
@@ -1405,7 +1399,7 @@ def main():
              'filename (relative to directory of list file) of a dynamic topography grid at a particular time. '
              'Second column containing associated time (in Ma).')
     
-    # Can optionally specify sea level as a filename or  model name (if using bundled data) but not both.
+    # Can optionally specify sea level as a filename or model name (if using bundled data) but not both.
     sea_level_argument_group = parser.add_mutually_exclusive_group()
     sea_level_argument_group.add_argument(
         '-slm', '--bundle_sea_level_model', type=str,
@@ -1441,6 +1435,16 @@ def main():
     else:
         age_grid_filename = args.age_grid_filename
     
+    # If rifting end time (and optional start time) was specified then we have a rifting period.
+    if args.rift_end_time is not None:
+        # Rifting start time is optional (can be None).
+        rifting_period = args.rift_start_time, args.rift_end_time
+    else:
+        # Rifting end time was not specified so rifting start time should not have been either.
+        if args.rift_start_time is not None:
+            raise ValueError('Cannot specify rift start time without also specifying rift end time')
+        rifting_period = None
+    
     # See if user overrode the total sediment thickness grid (because they know well site was drilled to basement depth).
     if args.no_total_sediment_thickness:
         total_sediment_thickness_filename = None
@@ -1494,7 +1498,7 @@ def main():
         sea_level_model,
         args.base_lithology_name,
         args.ocean_age_to_depth_model,
-        (args.rift_start_time, args.rift_end_time),
+        rifting_period,
         decompacted_columns,
         args.well_location,
         args.well_columns[0],  # well_bottom_age_column
@@ -1525,6 +1529,9 @@ if __name__ == '__main__':
     #
     # Importing this module (eg, 'import pybacktrack.backtrack') is fine though.
     #
+    # NOTE: We only warn instead of raising a runtime error because users of previous versions of pybacktrack
+    #       (prior to the introduction of the 'backtrack_cli' module) are likely still running the 'backtrack' module as a script.
+    #
     warnings.warn("Use 'python -m pybacktrack.backtrack_cli ...', instead of 'python -m pybacktrack.backtrack ...'.", DeprecationWarning)
     
     try:
